@@ -1,4 +1,4 @@
-{-# LANGUAGE TypeSynonymInstances, FlexibleInstances, CPP #-}
+{-# LANGUAGE TypeSynonymInstances, FlexibleInstances, CPP, ScopedTypeVariables, RankNTypes #-}
 
 module Language.Haskell.GhcMod.Gap (
     Language.Haskell.GhcMod.Gap.ClsInst
@@ -24,6 +24,8 @@ module Language.Haskell.GhcMod.Gap (
   , HasType(..)
   , errorMsgSpan
   , typeForUser
+  , nameForUser
+  , occNameForUser
   , deSugar
   , showDocWith
   , GapThing(..)
@@ -31,6 +33,14 @@ module Language.Haskell.GhcMod.Gap (
   , fileModSummary
   , WarnFlags
   , emptyWarnFlags
+  , benchmarkBuildInfo
+  , benchmarkTargets
+  , toModuleString
+  , GLMatch
+  , GLMatchI
+  , getClass
+  , occName
+  , setFlags
   ) where
 
 import Control.Applicative hiding (empty)
@@ -44,16 +54,19 @@ import Desugar (deSugarExpr)
 import DynFlags
 import ErrUtils
 import FastString
+import GhcMonad
 import HscTypes
 import Language.Haskell.GhcMod.GHCChoice
-import Language.Haskell.GhcMod.Types hiding (convert)
+import Language.Haskell.GhcMod.Types
 import NameSet
+import OccName
 import Outputable
 import PprTyThing
 import StringBuffer
 import TcType
 import Var (varType)
 
+import qualified Distribution.PackageDescription as P
 import qualified InstEnv
 import qualified Pretty
 import qualified StringBuffer as SB
@@ -72,10 +85,12 @@ import GHC hiding (ClsInst)
 import GHC hiding (Instance)
 import Control.Arrow hiding ((<+>))
 import Data.Convertible
+import RdrName (rdrNameOcc)
 #endif
 
 #if __GLASGOW_HASKELL__ >= 704
 import qualified Data.IntSet as I (IntSet, empty)
+import qualified Distribution.ModuleName as M (ModuleName,toFilePath)
 #endif
 
 ----------------------------------------------------------------
@@ -148,7 +163,7 @@ getSrcFile _ = Nothing
 
 ----------------------------------------------------------------
 
-toStringBuffer :: [String] -> Ghc StringBuffer
+toStringBuffer :: GhcMonad m => [String] -> m StringBuffer
 #if __GLASGOW_HASKELL__ >= 702
 toStringBuffer = return . stringToStringBuffer . unlines
 #else
@@ -171,13 +186,13 @@ fOptions = [option | (option,_,_,_) <- fFlags]
 ----------------------------------------------------------------
 ----------------------------------------------------------------
 
-fileModSummary :: FilePath -> Ghc ModSummary
+fileModSummary :: GhcMonad m => FilePath -> m ModSummary
 fileModSummary file = do
     mss <- getModuleGraph
     let [ms] = filter (\m -> ml_hs_file (ms_location m) == Just file) mss
     return ms
 
-withContext :: Ghc a -> Ghc a
+withContext :: GhcMonad m => m a -> m a
 withContext action = gbracket setup teardown body
   where
     setup = getContext
@@ -276,8 +291,10 @@ class HasType a where
 
 instance HasType (LHsBind Id) where
 #if __GLASGOW_HASKELL__ >= 708
-    getType _ (L spn FunBind{fun_matches = MG _ in_tys out_typ}) = return $ Just (spn, typ)
-      where typ = mkFunTys in_tys out_typ
+    getType _ (L spn FunBind{fun_matches = m}) = return $ Just (spn, typ)
+      where in_tys = mg_arg_tys m
+            out_typ = mg_res_ty m
+            typ = mkFunTys in_tys out_typ
 #else
     getType _ (L spn FunBind{fun_matches = MatchGroup _ typ}) = return $ Just (spn, typ)
 #endif
@@ -293,7 +310,7 @@ filterOutChildren get_thing xs
   where
     implicits = mkNameSet [getName t | x <- xs, t <- implicitTyThings (get_thing x)]
 
-infoThing :: String -> Ghc SDoc
+infoThing :: GhcMonad m => String -> m SDoc
 infoThing str = do
     names <- parseName str
 #if __GLASGOW_HASKELL__ >= 708
@@ -345,6 +362,12 @@ typeForUser = pprTypeForUser
 typeForUser = pprTypeForUser False
 #endif
 
+nameForUser :: Name -> SDoc
+nameForUser = pprOccName . getOccName
+
+occNameForUser :: OccName -> SDoc
+occNameForUser = pprOccName
+
 deSugar :: TypecheckedModule -> LHsExpr Id -> HscEnv
          -> IO (Maybe CoreExpr)
 #if __GLASGOW_HASKELL__ >= 708
@@ -386,3 +409,67 @@ type WarnFlags = [WarningFlag]
 emptyWarnFlags :: WarnFlags
 emptyWarnFlags = []
 #endif
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+benchmarkBuildInfo :: P.PackageDescription -> [P.BuildInfo]
+#if __GLASGOW_HASKELL__ >= 704
+benchmarkBuildInfo pd = map P.benchmarkBuildInfo $ P.benchmarks pd
+#else
+benchmarkBuildInfo pd = []
+#endif
+
+benchmarkTargets :: P.PackageDescription -> [String]
+#if __GLASGOW_HASKELL__ >= 704
+benchmarkTargets pd = map toModuleString $ concatMap P.benchmarkModules $ P.benchmarks pd
+#else
+benchmarkTargets = []
+#endif
+
+toModuleString :: M.ModuleName -> String
+toModuleString mn = fromFilePath $ M.toFilePath mn
+  where
+    fromFilePath :: FilePath -> String
+    fromFilePath fp = map (\c -> if c=='/' then '.' else c) fp
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+#if __GLASGOW_HASKELL__ >= 708
+type GLMatch = LMatch RdrName (LHsExpr RdrName)
+type GLMatchI = LMatch Id (LHsExpr Id)
+#else
+type GLMatch = LMatch RdrName
+type GLMatchI = LMatch Id
+#endif
+
+getClass :: [LInstDecl Name] -> Maybe (Name, SrcSpan)
+#if __GLASGOW_HASKELL__ >= 708
+-- Instance declarations of sort 'instance F (G a)'
+getClass [L loc (ClsInstD (ClsInstDecl {cid_poly_ty = (L _ (HsForAllTy _ _ _ (L _ (HsAppTy (L _ (HsTyVar className)) _))))}))] = Just (className, loc)
+-- Instance declarations of sort 'instance F G' (no variables)
+getClass [L loc (ClsInstD (ClsInstDecl {cid_poly_ty = (L _ (HsAppTy (L _ (HsTyVar className)) _))}))] = Just (className, loc)
+#elif __GLASGOW_HASKELL__ >= 706
+getClass [L loc (ClsInstD (L _ (HsForAllTy _ _ _ (L _ (HsAppTy (L _ (HsTyVar className)) _)))) _ _ _)] = Just (className, loc)
+getClass[L loc (ClsInstD (L _ (HsAppTy (L _ (HsTyVar className)) _)) _ _ _)] = Just (className, loc)
+#else
+getClass [L loc (InstDecl (L _ (HsForAllTy _ _ _ (L _ (HsAppTy (L _ (HsTyVar className)) _)))) _ _ _)] = Just (className, loc)
+getClass [L loc (InstDecl (L _ (HsAppTy (L _ (HsTyVar className)) _)) _ _ _)] = Just (className, loc)
+#endif
+getClass _ = Nothing
+
+#if __GLASGOW_HASKELL__ < 706
+occName :: RdrName -> OccName
+occName = rdrNameOcc
+#endif
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+setFlags :: DynFlags -> DynFlags
+#if __GLASGOW_HASKELL__ >= 708
+setFlags df = df `gopt_unset` Opt_SpecConstr -- consume memory if -O2
+#else
+setFlags = id
+#endif
