@@ -9,6 +9,7 @@
 ;;; Code:
 
 (require 'ghc-func)
+(require 'ghc-rewrite)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -126,7 +127,7 @@ unloaded modules are loaded")
   (interactive)
   (if (ghc-should-scroll)
       (ghc-scroll-completion-buffer)
-    (ghc-try-complete)))
+      (ghc-try-complete)))
 
 (defun ghc-should-scroll ()
   (let ((window (ghc-completion-window)))
@@ -168,7 +169,7 @@ unloaded modules are loaded")
 	    (ghc-reset-window-configuration)
 	  (ghc-save-window-configuration)
 	  (with-output-to-temp-buffer ghc-completion-buffer-name
-	    (display-completion-list list pattern))))))))
+	    (display-completion-list list))))))))
 
 (defun ghc-save-window-configuration ()
   (unless (get-buffer-window ghc-completion-buffer-name)
